@@ -20,55 +20,59 @@
 
 #define QT_FEATURE_reduce_exports 1
 
-#define QT_FEATURE_reduce_relocations 1
+#define QT_FEATURE_no_direct_extern_access -1
 
-#define QT_FEATURE_sse2 1
+#define QT_FEATURE_sse2 -1
 
-#define QT_FEATURE_sse3 1
+#define QT_FEATURE_sse3 -1
 
-#define QT_FEATURE_ssse3 1
+#define QT_FEATURE_ssse3 -1
 
-#define QT_FEATURE_sse4_1 1
+#define QT_FEATURE_sse4_1 -1
 
-#define QT_FEATURE_sse4_2 1
+#define QT_FEATURE_sse4_2 -1
 
-#define QT_FEATURE_avx 1
+#define QT_FEATURE_avx -1
 
-#define QT_FEATURE_f16c 1
+#define QT_FEATURE_f16c -1
 
-#define QT_FEATURE_avx2 1
+#define QT_FEATURE_avx2 -1
 
-#define QT_FEATURE_avx512f 1
+#define QT_FEATURE_avx512f -1
 
-#define QT_FEATURE_avx512er 1
+#define QT_FEATURE_avx512er -1
 
-#define QT_FEATURE_avx512cd 1
+#define QT_FEATURE_avx512cd -1
 
-#define QT_FEATURE_avx512pf 1
+#define QT_FEATURE_avx512pf -1
 
-#define QT_FEATURE_avx512dq 1
+#define QT_FEATURE_avx512dq -1
 
-#define QT_FEATURE_avx512bw 1
+#define QT_FEATURE_avx512bw -1
 
-#define QT_FEATURE_avx512vl 1
+#define QT_FEATURE_avx512vl -1
 
-#define QT_FEATURE_avx512ifma 1
+#define QT_FEATURE_avx512ifma -1
 
-#define QT_FEATURE_avx512vbmi 1
+#define QT_FEATURE_avx512vbmi -1
 
-#define QT_FEATURE_aesni 1
+#define QT_FEATURE_avx512vbmi2 -1
 
-#define QT_FEATURE_rdrnd 1
+#define QT_FEATURE_aesni -1
 
-#define QT_FEATURE_rdseed 1
+#define QT_FEATURE_vaes -1
 
-#define QT_FEATURE_shani 1
+#define QT_FEATURE_rdrnd -1
+
+#define QT_FEATURE_rdseed -1
+
+#define QT_FEATURE_shani -1
 
 #define QT_FEATURE_mips_dsp -1
 
 #define QT_FEATURE_mips_dspr2 -1
 
-#define QT_FEATURE_neon -1
+#define QT_FEATURE_neon 1
 
 #define QT_FEATURE_arm_crc32 -1
 
@@ -86,8 +90,6 @@
 
 #define QT_FEATURE_system_zlib 1
 
-#define QT_FEATURE_zstd 1
-
 #define QT_FEATURE_stdlib_libcpp -1
 
 #define QT_FEATURE_dbus 1
