@@ -1,38 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the Qt Gui module
-**
-** $QT_BEGIN_LICENSE:LGPL3$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QRHIGLES2_P_H
 #define QRHIGLES2_P_H
@@ -52,8 +19,11 @@
 #include "qrhi_p_p.h"
 #include "qshaderdescription_p.h"
 #include <qopengl.h>
+#include <QByteArray>
 #include <QSurface>
 
+#include <QtCore/private/qduplicatetracker_p.h>
+
 QT_BEGIN_NAMESPACE
 
 class QOpenGLExtensions;
@@ -71,7 +41,7 @@ struct QGles2Buffer : public QRhiBuffer
     int nonZeroSize = 0;
     GLuint buffer = 0;
     GLenum targetForDataOps;
-    char *data = nullptr;
+    QByteArray data;
     enum Access {
         AccessNone,
         AccessVertex,
@@ -97,11 +67,14 @@ struct QGles2RenderBuffer : public QRhiRenderBuffer
     ~QGles2RenderBuffer();
     void destroy() override;
     bool create() override;
+    bool createFrom(NativeRenderBuffer src) override;
     QRhiTexture::Format backingFormat() const override;
 
     GLuint renderbuffer = 0;
     GLuint stencilRenderbuffer = 0; // when packed depth-stencil not supported
     int samples;
+    bool owns = true;
+    uint generation = 0;
     friend class QRhiGles2;
 };
 
@@ -111,6 +84,7 @@ struct QGles2SamplerData
     GLenum glmagfilter = 0;
     GLenum glwraps = 0;
     GLenum glwrapt = 0;
+    GLenum glwrapr = 0;
     GLenum gltexcomparefunc = 0;
 };
 
@@ -120,6 +94,7 @@ inline bool operator==(const QGles2SamplerData &a, const QGles2SamplerData &b)
             && a.glmagfilter == b.glmagfilter
             && a.glwraps == b.glwraps
             && a.glwrapt == b.glwrapt
+            && a.glwrapr == b.glwrapr
             && a.gltexcomparefunc == b.gltexcomparefunc;
 }
 
@@ -130,8 +105,8 @@ inline bool operator!=(const QGles2SamplerData &a, const QGles2SamplerData &b)
 
 struct QGles2Texture : public QRhiTexture
 {
-    QGles2Texture(QRhiImplementation *rhi, Format format, const QSize &pixelSize,
-                  int sampleCount, Flags flags);
+    QGles2Texture(QRhiImplementation *rhi, Format format, const QSize &pixelSize, int depth,
+                  int arraySize, int sampleCount, Flags flags);
     ~QGles2Texture();
     void destroy() override;
     bool create() override;
@@ -149,7 +124,7 @@ struct QGles2Texture : public QRhiTexture
     GLenum gltype;
     QGles2SamplerData samplerState;
     bool specified = false;
-    bool compressedAtlasBuilt = false;
+    bool zeroInitialized = false;
     int mipLevelCount = 0;
 
     enum Access {
@@ -190,6 +165,8 @@ struct QGles2RenderPassDescriptor : public QRhiRenderPassDescriptor
     ~QGles2RenderPassDescriptor();
     void destroy() override;
     bool isCompatible(const QRhiRenderPassDescriptor *other) const override;
+    QRhiRenderPassDescriptor *newCompatibleRenderPassDescriptor() const override;
+    QVector<quint32> serializedFormat() const override;
 };
 
 struct QGles2RenderTargetData
@@ -203,12 +180,13 @@ struct QGles2RenderTargetData
     int colorAttCount = 0;
     int dsAttCount = 0;
     bool srgbUpdateAndBlend = false;
+    QRhiRenderTargetAttachmentTracker::ResIdList currentResIdList;
 };
 
-struct QGles2ReferenceRenderTarget : public QRhiRenderTarget
+struct QGles2SwapChainRenderTarget : public QRhiSwapChainRenderTarget
 {
-    QGles2ReferenceRenderTarget(QRhiImplementation *rhi);
-    ~QGles2ReferenceRenderTarget();
+    QGles2SwapChainRenderTarget(QRhiImplementation *rhi, QRhiSwapChain *swapchain);
+    ~QGles2SwapChainRenderTarget();
     void destroy() override;
 
     QSize pixelSize() const override;
@@ -242,6 +220,7 @@ struct QGles2ShaderResourceBindings : public QRhiShaderResourceBindings
     ~QGles2ShaderResourceBindings();
     void destroy() override;
     bool create() override;
+    void updateResources(UpdateFlags flags) override;
 
     bool hasDynamicOffset = false;
     uint generation = 0;
@@ -263,7 +242,9 @@ Q_DECLARE_TYPEINFO(QGles2UniformDescription, Q_RELOCATABLE_TYPE);
 struct QGles2SamplerDescription
 {
     int glslLocation;
-    int binding;
+    int combinedBinding;
+    int tbinding;
+    int sbinding;
 };
 
 Q_DECLARE_TYPEINFO(QGles2SamplerDescription, Q_RELOCATABLE_TYPE);
@@ -434,17 +415,20 @@ struct QGles2CommandBuffer : public QRhiCommandBuffer
                 int size;
             } getBufferSubData;
             struct {
+                GLenum srcTarget;
                 GLenum srcFaceTarget;
                 GLuint srcTexture;
                 int srcLevel;
                 int srcX;
                 int srcY;
+                int srcZ;
                 GLenum dstTarget;
                 GLuint dstTexture;
                 GLenum dstFaceTarget;
                 int dstLevel;
                 int dstX;
                 int dstY;
+                int dstZ;
                 int w;
                 int h;
             } copyTex;
@@ -456,6 +440,7 @@ struct QGles2CommandBuffer : public QRhiCommandBuffer
                 QRhiTexture::Format format;
                 GLenum readTarget;
                 int level;
+                int slice3D;
             } readPixels;
             struct {
                 GLenum target;
@@ -464,11 +449,13 @@ struct QGles2CommandBuffer : public QRhiCommandBuffer
                 int level;
                 int dx;
                 int dy;
+                int dz;
                 int w;
                 int h;
                 GLenum glformat;
                 GLenum gltype;
                 int rowStartAlign;
+                int rowLength;
                 const void *data; // must come from retainImage()
             } subImage;
             struct {
@@ -479,6 +466,7 @@ struct QGles2CommandBuffer : public QRhiCommandBuffer
                 GLenum glintformat;
                 int w;
                 int h;
+                int depth;
                 int size;
                 const void *data; // must come from retainData()
             } compressedImage;
@@ -489,6 +477,7 @@ struct QGles2CommandBuffer : public QRhiCommandBuffer
                 int level;
                 int dx;
                 int dy;
+                int dz;
                 int w;
                 int h;
                 GLenum glintformat;
@@ -502,6 +491,7 @@ struct QGles2CommandBuffer : public QRhiCommandBuffer
                 GLenum target;
                 GLuint texture;
                 int dstLevel;
+                int dstLayer;
             } blitFromRb;
             struct {
                 GLenum target;
@@ -576,6 +566,8 @@ struct QGles2CommandBuffer : public QRhiCommandBuffer
         float polyOffsetFactor;
         float polyOffsetUnits;
         float lineWidth;
+        int cpCount;
+        GLenum polygonMode;
         void reset() { valid = false; }
         struct {
             // not part of QRhiGraphicsPipeline but used by setGraphicsPipeline()
@@ -700,13 +692,14 @@ struct QGles2SwapChain : public QRhiSwapChain
     QRhiRenderTarget *currentFrameRenderTarget() override;
 
     QSize surfacePixelSize() override;
+    bool isFormatSupported(Format f) override;
 
     QRhiRenderPassDescriptor *newCompatibleRenderPassDescriptor() override;
     bool createOrResize() override;
 
     QSurface *surface = nullptr;
     QSize pixelSize;
-    QGles2ReferenceRenderTarget rt;
+    QGles2SwapChainRenderTarget rt;
     QGles2CommandBuffer cb;
     int frameCount = 0;
 };
@@ -732,6 +725,8 @@ public:
                                          QRhiTexture::Format backingFormatHint) override;
     QRhiTexture *createTexture(QRhiTexture::Format format,
                                const QSize &pixelSize,
+                               int depth,
+                               int arraySize,
                                int sampleCount,
                                QRhiTexture::Flags flags) override;
     QRhiSampler *createSampler(QRhiSampler::Filter magFilter,
@@ -811,11 +806,15 @@ public:
     bool isFeatureSupported(QRhi::Feature feature) const override;
     int resourceLimit(QRhi::ResourceLimit limit) const override;
     const QRhiNativeHandles *nativeHandles() override;
-    void sendVMemStatsToProfiler() override;
+    QRhiDriverInfo driverInfo() const override;
+    QRhiMemAllocStats graphicsMemoryAllocationStatistics() override;
     bool makeThreadLocalNativeContextCurrent() override;
     void releaseCachedResources() override;
     bool isDeviceLost() const override;
 
+    QByteArray pipelineCacheData() override;
+    void setPipelineCacheData(const QByteArray &data) override;
+
     bool ensureContext(QSurface *surface = nullptr) const;
     void executeDeferredReleases();
     void trackedBufferBarrier(QGles2CommandBuffer *cbD, QGles2Buffer *bufD, QGles2Buffer::Access access);
@@ -833,6 +832,9 @@ public:
                                 QRhiPassResourceTracker::TextureStage stage);
     void executeCommandBuffer(QRhiCommandBuffer *cb);
     void executeBindGraphicsPipeline(QGles2CommandBuffer *cbD, QGles2GraphicsPipeline *psD);
+    void bindCombinedSampler(QGles2CommandBuffer *cbD, QGles2Texture *texD, QGles2Sampler *samplerD,
+                             void *ps, uint psGeneration, int glslLocation,
+                             int *texUnit, bool *activeTexUnitAltered);
     void bindShaderResources(QGles2CommandBuffer *cbD,
                              QRhiGraphicsPipeline *maybeGraphicsPs, QRhiComputePipeline *maybeComputePs,
                              QRhiShaderResourceBindings *srb,
@@ -841,39 +843,47 @@ public:
                                                    bool *wantsColorClear = nullptr, bool *wantsDsClear = nullptr);
     void enqueueBarriersForPass(QGles2CommandBuffer *cbD);
     int effectiveSampleCount(int sampleCount) const;
-    QByteArray shaderSource(const QRhiShaderStage &shaderStage, int *glslVersion);
-    bool compileShader(GLuint program, const QRhiShaderStage &shaderStage, int *glslVersion);
+    QByteArray shaderSource(const QRhiShaderStage &shaderStage, QShaderVersion *shaderVersion);
+    bool compileShader(GLuint program, const QRhiShaderStage &shaderStage, QShaderVersion *shaderVersion);
     bool linkProgram(GLuint program);
     void registerUniformIfActive(const QShaderDescription::BlockVariable &var,
                                  const QByteArray &namePrefix, int binding, int baseOffset,
                                  GLuint program,
-                                 QSet<int> *activeUniformLocations,
+                                 QDuplicateTracker<int, 256> *activeUniformLocations,
                                  QGles2UniformDescriptionVector *dst);
     void gatherUniforms(GLuint program, const QShaderDescription::UniformBlock &ub,
-                        QSet<int> *activeUniformLocations, QGles2UniformDescriptionVector *dst);
+                        QDuplicateTracker<int, 256> *activeUniformLocations, QGles2UniformDescriptionVector *dst);
     void gatherSamplers(GLuint program, const QShaderDescription::InOutVariable &v,
                         QGles2SamplerDescriptionVector *dst);
+    void gatherGeneratedSamplers(GLuint program,
+                                 const QShader::SeparateToCombinedImageSamplerMapping &mapping,
+                                 QGles2SamplerDescriptionVector *dst);
+    void sanityCheckVertexFragmentInterface(const QShaderDescription &vsDesc, const QShaderDescription &fsDesc);
     bool isProgramBinaryDiskCacheEnabled() const;
 
-    enum DiskCacheResult {
-        DiskCacheHit,
-        DiskCacheMiss,
-        DiskCacheError
+    enum ProgramCacheResult {
+        ProgramCacheHit,
+        ProgramCacheMiss,
+        ProgramCacheError
     };
-    DiskCacheResult tryLoadFromDiskCache(const QRhiShaderStage *stages,
-                                         int stageCount,
-                                         GLuint program,
-                                         const QVector<QShaderDescription::InOutVariable> &inputVars,
-                                         QByteArray *cacheKey);
+    ProgramCacheResult tryLoadFromDiskOrPipelineCache(const QRhiShaderStage *stages,
+                                                      int stageCount,
+                                                      GLuint program,
+                                                      const QVector<QShaderDescription::InOutVariable> &inputVars,
+                                                      QByteArray *cacheKey);
     void trySaveToDiskCache(GLuint program, const QByteArray &cacheKey);
+    void trySaveToPipelineCache(GLuint program, const QByteArray &cacheKey, bool force = false);
 
+    QRhi::Flags rhiFlags;
     QOpenGLContext *ctx = nullptr;
     bool importedContext = false;
     QSurfaceFormat requestedFormat;
     QSurface *fallbackSurface = nullptr;
     QWindow *maybeWindow = nullptr;
-    mutable bool needsMakeCurrent = false;
+    QOpenGLContext *maybeShareContext = nullptr;
+    mutable bool needsMakeCurrentDueToSwap = false;
     QOpenGLExtensions *f = nullptr;
+    void (QOPENGLF_APIENTRYP glPolygonMode) (GLenum, GLenum) = nullptr;
     uint vao = 0;
     struct Caps {
         Caps()
@@ -882,11 +892,15 @@ public:
               maxTextureSize(2048),
               maxDrawBuffers(4),
               maxSamples(16),
+              maxTextureArraySize(0),
               maxThreadGroupsPerDimension(0),
               maxThreadsPerThreadGroup(0),
               maxThreadGroupsX(0),
               maxThreadGroupsY(0),
               maxThreadGroupsZ(0),
+              maxUniformVectors(4096),
+              maxVertexInputs(8),
+              maxVertexOutputs(8),
               msaaRenderBuffer(false),
               multisampledTexture(false),
               npotTextureFull(true),
@@ -897,6 +911,7 @@ public:
               r8Format(false),
               r16Format(false),
               floatFormats(false),
+              rgb10Formats(false),
               depthTexture(false),
               packedDepthStencil(false),
               needsDepthStencilCombinedAttach(false),
@@ -914,18 +929,26 @@ public:
               nonBaseLevelFramebufferTexture(false),
               texelFetch(false),
               intAttributes(true),
-              screenSpaceDerivatives(false)
+              screenSpaceDerivatives(false),
+              programBinary(false),
+              texture3D(false),
+              tessellation(false),
+              geometryShader(false)
         { }
         int ctxMajor;
         int ctxMinor;
         int maxTextureSize;
         int maxDrawBuffers;
         int maxSamples;
+        int maxTextureArraySize;
         int maxThreadGroupsPerDimension;
         int maxThreadsPerThreadGroup;
         int maxThreadGroupsX;
         int maxThreadGroupsY;
         int maxThreadGroupsZ;
+        int maxUniformVectors;
+        int maxVertexInputs;
+        int maxVertexOutputs;
         // Multisample fb and blit are supported (GLES 3.0 or OpenGL 3.x). Not
         // the same as multisample textures!
         uint msaaRenderBuffer : 1;
@@ -938,6 +961,7 @@ public:
         uint r8Format : 1;
         uint r16Format : 1;
         uint floatFormats : 1;
+        uint rgb10Formats : 1;
         uint depthTexture : 1;
         uint packedDepthStencil : 1;
         uint needsDepthStencilCombinedAttach : 1;
@@ -956,11 +980,16 @@ public:
         uint texelFetch : 1;
         uint intAttributes : 1;
         uint screenSpaceDerivatives : 1;
+        uint programBinary : 1;
+        uint texture3D : 1;
+        uint tessellation : 1;
+        uint geometryShader : 1;
     } caps;
     QGles2SwapChain *currentSwapChain = nullptr;
-    QList<GLint> supportedCompressedFormats;
+    QSet<GLint> supportedCompressedFormats;
     mutable QList<int> supportedSampleCountList;
     QRhiGles2NativeHandles nativeHandlesStruct;
+    QRhiDriverInfo driverInfoStruct;
     mutable bool contextLost = false;
 
     struct DeferredReleaseEntry {
@@ -1000,6 +1029,12 @@ public:
     } ofr;
 
     QHash<QRhiShaderStage, uint> m_shaderCache;
+
+    struct PipelineCacheData {
+        quint32 format;
+        QByteArray data;
+    };
+    QHash<QByteArray, PipelineCacheData> m_pipelineCache;
 };
 
 Q_DECLARE_TYPEINFO(QRhiGles2::DeferredReleaseEntry, Q_RELOCATABLE_TYPE);
